%colonModel: Model to simulate colon cancer
%
%Main scheme of differentiation:
%   N0 -> N1 -> N2 -> 
% 
%Variables
%   N0: colon stem cells.
%   N1: semi-differentiated colon cells.
%   N2: fully-differentiated colon cells.
% 
%Time delay (entra.del):
%   Time delay for the transit through different cell type in hours.
%   Input variable "entra.del" introduce a specific time delay, in hour, to be
%   simulated.
%   To simulate different time delay in the same figure introduce the
%   different values in an array in the variable. P.e. entra.del = [0 24 24*4 24*8].
%   For simulate without delay let the variable equal to zero.
% 
%Biological parameters:
%   Biological parameters can be changed in "Biological parameters"
%   section.
%   in the script. References for default values can be found there.
% 
%Mathematical integration parameters:
%   To modify integration interval and variables shown figures go to the 
%   script to "Mathematical integration parameters" section.
% 
%Mathematical model parameters
%   Biological meaning and references for the values of the parameters of
%   the model:
%       alpha1: Death rate of stem cells (hours^-1), assume 0 value
%       alpha2: Differentiation rate of stem cells into semi-differentiated cells (1 division per 24 hours in a cycle of 7 differentiation divisions per 15 divisions), from CellCycle ref.
%       alpha3: Growth rate of stem cells cells (1 division per 24 hours in a cycle of 7 differentiation divisions per 15 divisions), from CellCycle ref. 
%       beta1: Death rate of semi-differentiated cells (hours^-1), assume 0 value
%       beta2: Differentiation rate of semi-differentiated cells into fully-differentiated cells (1 division per 12 hours in a cycle of 7 differentiation divisions per 15 divisions), from CellCycle ref.
%       beta3: Growth rate of semi-differentiated cells (1 division per 12 hours in a cycle of 7 differentiation divisions per 15 divisions), from CellCycle ref.
%       gamma: Death rate of fully-differentiated cells (hours^-1), from CellCycle ref. 0.1390, Development ref.: N2(t=0) = 104 cells, hand fitting: 0.1390*0.2941
%       k0: Response rate of the feedback for differentiation of stem cells (hours^-1), from Development ref.: N0(t=0) = 16 cells, calculate using steady state formula from CellCycle ref.
%       k1: Response rate of the feedback for differentiation of semi-differentiated cells (hours^-1), from Development ref.: N1(t=0) = 130 cells, calculate using steady state formula from CellCycle ref.
%       m0: Feedback satutation dimensionless parameter for differentiation of stem cells, assume 0 value, if in the tumor there are a low amount of stem cells
%       m1: Feedback satutation dimensionless parameter for differentiation of semi-differentiated cells, from CellCycle ref. value 0.0004, low sensitive value from CellCycle ref. analysis
%       l0: Rate order of the response rate, from CellCycle ref. value 2, from IntJBio ref. value 10.0000
%       l1: Rate order for the saturation of the response rate, from CellCycle ref. value 1, from IntJBio ref. value 10.0000
% 
%References:
%   Article of IntJBio: http://www.clbme.bas.bg/bioautomation/2015/vol_19.s1/files/19.s1_03.pdf
%   Article of PNAS: http://www.pnas.org/content/104/10/4008.full
%   Article of CellCycle: http://www.ncbi.nlm.nih.gov/pmc/articles/PMC2430384/
%   Article of Development: http://dev.biologists.org/content/110/4/1001.long
%   Article of Nature: http://www.nature.com/nrm/journal/v15/n1/pdf/nrm3721.pdf
%   Book of Daskalaki: https://books.google.de/books?id=FxFfukfbl2EC&pg=PR17&lpg=PR17&dq=Medical+Advancements+in+Aging+and+Regenerative+Technologies:&source=bl&ots=NuvcXXmt3P&sig=OrALRcH5hKN13RDwM0OFoD_M0EQ&hl=es&sa=X&ved=0CDAQ6AEwAmoVChMI4pbE3pyayQIVi1YaCh0CoAGN
% 
%Guido Santos, ver. 1.3. November, 24th 2015

%%
%Main function
function [sol1] = colonModel(in,T,co,label)

    %Biological parameters
    entra.del     = 24; %Biologogical delay between the activation of division and the cumination, in hours (24).
    entra.exp     = 0; %Value 1 for increase exponent to the value shown in entra.expVal, 0 for l = 2 and l0 = 1 from Development ref.
    entra.expVal  = 10; %Value for the exponents (l = l0 = 10)
    entra.K1      = 1; %Normalization parameter for variable N0, the parameter multiplies the value of the variables (47 for l = l0 = 10). 
    entra.N0      = 16; %Number of N0 cells at steady state, Development ref.: 16 
    entra.N1      = 130; %Number of N1 cells at steady state, Development ref.: 130
    entra.N2      = 104; %Number of N1 cells at steady state, Development ref.: 104
    entra.ciclo   = 15; %Number of divisions for each cycle of asymmetric divisions, Nature ref.: 6 months to fill a crypt from 1 cell (15 days)
    entra.ndif    = 7; %Number of divisions inside assymetric cycle in which occurs differentiation division, Nature ref.: 6 months to fill a crypt from 1 cell (7 days)
    entra.tdiv0   = 24; %Cell cycle time for stem cells in hours, Development ref.: 1 division per 24 hours
    entra.tdiv1   = 12; %Cell cycle time for semi-differentiated in hours, Development ref.: 1 division per 12 hours
    entra.init    = [16;130;104]*entra.K1; %Initial conditions in number of cells: [N0;N1;N2]. Steady state: [16;130;104] from Development ref.
    entra.mutar   = 0; %If 1 then simulates mutation, if 0 it doesn't
    entra.cambio  = 5*[1]; %Proportion change on parameters after each mutation (1.9 and 4.7 times [1])
    entra.mutTime = [1e3]; %Time in hours for the mutation being produces (same length than entra.cambio), search for the value. P.e. [1e3 2e3 3e3 4e3]
%     entra.chem    = 0*0.1; %Chemotherapy killing rate, depends on the growth rate
    
    %Mathematical integration parameters
    entra.tspan    = 0:150*24; %Interval of integration in hours. P.e.: 0:250*24
    entra.neqs     = 50; %Number of equations for the definition of the delay (default: 50)
    entra.paintVar = 0; %Introduce the indexes of the variables to be painted. 1:N0, 2:N1, 3:N2; 0 for painting the sum
    entra.sensCalc = 0; %Calculate the sensitivities for the model parameters, 1 calculate; 0 no calculate
    entra.figura   = 1; %Create figure with the results if 1, if 0 not.
    
    %Mathematical model variables and parameters
        entra.vars      = {'N0','N1','N2'}; %Stem cells, semi-differentiate cells and fully differentiate cells
        entra.names(1)  = {'alpha1'}; %Death rate of stem cells (hours^-1), assume 0 value   
        entra.names(2)  = {'alpha2'}; %Differentiation rate of stem cells into semi-differentiated cells (1 division per 24 hours in a cycle of 7 differentiation divisions per 15 divisions), from CellCycle ref.
        entra.names(3)  = {'alpha3'}; %Growth rate of stem cells cells (1 division per 24 hours in a cycle of 7 differentiation divisions per 15 divisions), from CellCycle ref. 
        entra.names(4)  = {'beta1'}; %Death rate of semi-differentiated cells (hours^-1), assume 0 value
        entra.names(5)  = {'beta2'}; %Differentiation rate of semi-differentiated cells into fully-differentiated cells (1 division per 12 hours in a cycle of 7 differentiation divisions per 15 divisions), from CellCycle ref.
        entra.names(6)  = {'beta3'}; % Growth rate of semi-differentiated cells (1 division per 12 hours in a cycle of 7 differentiation divisions per 15 divisions), from CellCycle ref.
        entra.names(7)  = {'gamma'}; % Death rate of fully-differentiated cells (hours^-1), from Development ref.: N2(t=0) = 104 cells, calculate using steady state formula from Daskalaki ref.
        entra.names(8)  = {'k0'}; % Response rate of the feedback for differentiation of stem cells (hours^-1), from Development ref.: N0(t=0) = 16 cells, calculate using steady state formula from CellCycle ref.
        entra.names(9)  = {'k1'}; % Response rate of the feedback for differentiation of semi-differentiated cells (hours^-1), from Development ref.: N1(t=0) = 130 cells, calculate using steady state formula from CellCycle ref.
        entra.names(10) = {'m0'}; % Feedback satutation dimensionless parameter for differentiation of stem cells, assume 0 value, if in the tumor there are a low amount of stem cells
        entra.names(11) = {'m1'}; % Feedback satutation dimensionless parameter for differentiation of semi-differentiated cells, from CellCycle ref. value 0.0004, low sensitive value from CellCycle ref. analysis
        entra.names(12) = {'l0'}; % Rate order of the response rate, from CellCycle ref. value 2, from IntJBio ref. value 10.0000
        entra.names(13) = {'l1'}; % Rate order for the saturation of the response rate, from CellCycle ref. value 1, from IntJBio ref. value 10.0000
%% 

%Test if there is input
    A = exist('in','var');
    
    if (A == 0 || numel(in) == 0)
        in = ones(1,13); %Modify the parameter values
    end
    
    B = exist('T','var');
    
    if (B == 0 || numel(T) == 0)
        T = 0; 
    end
    
    C = exist('co','var');
    
    if (C == 0 || numel(co) == 0)
        co = 'k'; 
    end
    
    D = exist('label','var');
    
    if (D == 0 || numel(label) == 0)
        label = 1; 
    end
    
    entra.color = co;
    entra.mod   = in;
    entra.chem  = T*0.1; %Chemotherapy killing rate, depends on the growth rate
%%
    
%Model 1
    entra.condiciones = 1;
%     entra.mod([2,3,5,6]) = entra.mod([2,3,5,6])*0.2; %Simulated growth inhibition by drug
    entra.f           = label;
    entra.exp         = 0;
    entra.init        = [16;130;104];
    
%   Perturbation simulation
%     if (entra.mutar == 0 && entra.sensCalc == 0)
%         entra.init    = entra.init + 0.1*entra.init;
%     end
    
    entra             = configModel(entra);
    sol1              = [entra.t entra.y];
%%

% %Model 2     
%     entra.f           = 2;
%     entra.exp         = 1;
%     entra.init        = [1;0;0];
%     entra.figura      = 0;
%     entra             = configModel(entra);
%     finalCells        = mean(sum(entra.y(end - 10:end,:),2));
%     totalCells        = entra.N0 + entra.N1 + entra.N2;
%     entra.K1          = totalCells/finalCells;
%     entra.init        = entra.K1*mean(entra.y(end - 10:end,:))';
%     
%     
% %   Perturbation simulation
% %     if (entra.mutar == 0 && entra.sensCalc == 0) 
% %         entra.init    = entra.init + 0.1*entra.init;
% %     end
%     
%     entra.figura = 1;        
%     entra        = configModel(entra);
%     sol2         = [entra.t entra.y];
%
%%

% %Configuration of the model
% %     entra.condiciones = 10;
% %     nombresM          = {'Col1: exponent value','Col2: mutation threshold'};
% %     nombresS          = {'Col1: exponent value','Col2: sens value'};
% %     entra             = configModel(entra);%
% %     hold on
% %     for n = 1:entra.condiciones
% %         entra.f   = n; %Control the figure in which is plotted the simulation
% %         
% %         if n == 1
% %             display('(n = 1) Calculating the maximum mutation value before coloncancer for different values of the exponents...')
% %             entra.del      = 24;
% %             entra.exp      = 0;
% %             entra.expVal   = 1; 
% %             entra.K1       = 1; 
% %             entra.init     = entra.init*0.99; %Perturbate the initial conditions of the model
% %             mutar          = entra.mutar;
% %             entra.mutar    = 1;
% %             sensCalc       = entra.sensCalc;
% %             entra.sensCalc = 0;
% %             entra.cambio   = 1; 
% %             lastValue      = 0;
% %             
% %             while lastValue < 1e5
% %                 entra          = configModel(entra);
% %                 lastValue      = sum(entra.y(end,:));
% %                 entra.cambio   = entra.cambio*1.1;
% %             end
% %             
% %             entra.sensCalc = sensCalc;
% %             entra.mutar    = mutar;
% %             maxCambio(n,1) = entra.expVal;
% %             maxCambio(n,2) = entra.cambio;
% %             display('(n = 1) Calculating sensitivties of parameters for different values of the exponents...')
% %             mutar          = entra.mutar;
% %             entra.mutar    = 0;
% %             sensCalc       = entra.sensCalc;
% %             entra.sensCalc = 1;
% %             entra          = configModel(entra);
% %             
% %             if entra.sensCalc == 1
% %                 sensMed(n,1) = entra.expVal;
% %                 sensMed(n,2) = entra.sens;
% %             end
% %             
% %             entra.sensCalc = sensCalc;
% %             entra.mutar    = mutar;
% %             
% %         else
% %             entra.del      = 24;
% %             entra.exp      = 1;
% %             entra.expVal   = n - 1; %It works for values >= 4
% %             display('(n > 1) Calculating the normalization factor (entra.K1) and the initial conditions in order to fix the number of total cell to the normal condition...')
% %             entra.init     = [1;0;0];
% %             entra.K1       = 1; 
% %             figura         = entra.figura;
% %             entra.figura   = 0;
% %             mutar          = entra.mutar;
% %             entra.mutar    = 0;
% %             sensCalc       = entra.sensCalc;
% %             entra.sensCalc = 0;
% %             entra          = configModel(entra);
% %             entra.figura   = figura;
% %             entra.mutar    = mutar;
% %             entra.sensCalc = sensCalc;
% %             finalCells     = mean(sum(entra.y(end - 10:end,:),2));
% %             totalCells     = entra.N0 + entra.N1 + entra.N2;
% %             entra.K1       = totalCells/finalCells;
% %             display('(n > 1) Calculating the maximum mutation value before coloncancer for different values of the exponents...')
% %             entra.init     = entra.K1*mean(entra.y(end - 10:end,:))';
% %             mutar          = entra.mutar;
% %             entra.mutar    = 1;
% %             sensCalc       = entra.sensCalc;
% %             entra.sensCalc = 0;
% %             entra.cambio   = 1; 
% %             lastValue      = 0;
% %             
% %             while lastValue < 1e5
% %                 entra          = configModel(entra);
% %                 lastValue      = sum(entra.y(end,:));
% %                 entra.cambio   = entra.cambio*1.1;
% %             end
% %             
% %             entra.sensCalc = sensCalc;
% %             entra.mutar    = mutar;
% %             maxCambio(n,1) = entra.expVal;
% %             maxCambio(n,2) = entra.cambio;
% %             display('(n > 1) Calculating sensitivties of parameters for different values of the exponents...')
% %             mutar          = entra.mutar;
% %             entra.mutar    = 0;
% %             sensCalc       = entra.sensCalc;
% %             entra.sensCalc = 1;
% %             entra          = configModel(entra);
% %             
% %             if entra.sensCalc == 1
% %                 sensMed(n,1) = entra.expVal;
% %                 sensMed(n,2) = entra.sens;
% %             end
% %         
% %             entra.sensCalc = sensCalc;
% %             entra.mutar    = mutar;
% %         end
% %         
% %         display('Saving the results')
% %         save('MutExp.mat','maxCambio','entra','nombresM')
% %         save('SensExp.mat','sensMed','entra','nombresS')
% %     end
%%
end

%%
%Mathematical model of colon cancer definition
function dx = eqs(t,x,entra)

    %Prevent negative values
    x(x <= 0) = 0;

    %Simulate mutations
    if entra.mutar == 1
        
        for n = 1:length(entra.mutTime)
            
            if t > entra.mutTime(n)
                entra.par(6) = entra.par(6)*entra.cambio(n); %Increase value of the selected parameter
%                 entra.par(3) = entra.par(3)*entra.cambio(n); %Increase value of the selected parameter
            end
            
        end
        
    end
    
    %Mathematical model parameters definition
    alpha1 = entra.par(1)*entra.mod(1);
    alpha2 = entra.par(2)*entra.mod(2);
    alpha3 = entra.par(3)*entra.mod(3);
    beta1  = entra.par(4)*entra.mod(4);
    beta2  = entra.par(5)*entra.mod(5);
    beta3  = entra.par(6)*entra.mod(6);
    gamma  = entra.par(7)*entra.mod(7);
    k0     = entra.par(8)*entra.mod(8)/(entra.K1^entra.par(12));
    k1     = entra.par(9)*entra.mod(9)/(entra.K1^entra.par(12));
    m0     = entra.par(10)*entra.mod(10)/(entra.K1^entra.par(13));
    m1     = entra.par(11)*entra.mod(11)/(entra.K1^entra.par(13));
    l0     = entra.par(12)*entra.mod(12) + 1;
    l1     = entra.par(13)*entra.mod(13); 
    kchem  = entra.chem; %Chemotherapy killing rate
    
    %Only positive solutions
    x(x < 0) = 0;
    
    %Number of equations
    eqs = length(entra.init);
    neq = (length(x) - eqs)/2;
    
    %Variables
    N0  = x(1);
    N1  = x(2); 
    N2  = x(3); 
    N0d = x(neq + eqs); 
    N1d = x(end); 
    
    if prod(entra.delays) == 0
        entra.delay = 0;
    end
    
    if entra.delay == 0
        N0d = N0;
        N1d = N1;
    end
    
    %Differential equations
    dN0   = - (alpha1 + alpha2)*N0 + alpha3*N0d - ((k0*N0d^l0)/(1 + m0*N0d^l1)) - alpha3*kchem*N0;
    dN1   = alpha2*N0 - (beta1 + beta2)*N1 + beta3*N1d - ((k1*N1d^l0)/(1 + m1*N1d^l1)) + ((k0*N0d^l0)/(1 + m0*N0d^l1)) - beta3*kchem*N1;
    dN2   = beta2*N1 - gamma*N2 + ((k1*N1d^l0)/(1 + m1*N1d^l1));
    dx    = zeros(length(x),1);
    dx(1) = dN0;
    dx(2) = dN1;
    dx(3) = dN2;

    if prod(entra.delays) ~= 0
        %Delay in N0 clearance synthesis
        timedel1    = entra.delays(1);
        kdl1        = neq/timedel1;
        dx(eqs + 1) = kdl1*(N0 - x(eqs + 1));

        for n = 2:neq
            dx(eqs + n) = kdl1*(x(eqs - 1 + n) - x(eqs + n));
        end

        %Delay in N1 clearance synthesis
        timedel2          = entra.delays(2);
        kdl2              = neq/timedel2;
        dx(eqs + neq + 1) = kdl2*(N1 - x(eqs + neq + 1));

        for n = 2:neq
            dx(eqs + neq + n) = kdl2*(x(eqs + neq - 1 + n) - x(eqs + neq + n));
        end
    
    end
    
end

%%
%Configuration of the model
function entra = configModel(entra)

    %Time delay default configuration
    entra.delay = 1; %Assume there is an input
    
    if sum(entra.del) == 0
        entra.delay  = 0; %Remove delay if there is no input or its value is 0
    end
    
    %Model parameters (for the biological meaning and references see the section "Mathematical model parameters" 
    %in the main function
    entra.par(1)  = 0.0000;                                                         %alpha1
    entra.par(2)  = log(2^entra.ndif)/(entra.ciclo*entra.tdiv0);                    %alpha2 
    entra.par(3)  = log(2^(entra.ciclo - entra.ndif))/(entra.ciclo*entra.tdiv0);    %alpha3
    entra.par(4)  = 0.0000;                                                         %beta1
    entra.par(5)  = log(2^entra.ndif)/(entra.ciclo*entra.tdiv1);                    %beta2
    entra.par(6)  = log(2^(entra.ciclo - entra.ndif))/(entra.ciclo*entra.tdiv1);    %beta3
    entra.par(10) = 0.0000;                                                         %m0
    entra.par(11) = 0.0004;                                                         %m1
    entra.par(12) = 1.0000;                                                         %l0
    entra.par(13) = 1.0000;                                                         %l1
    
    if entra.exp == 1 %If it is 1 take parameters from IntJBio ref.
        entra.par(12) = entra.expVal - 1;                                                %l0 
%         entra.par(12) = entra.expVal; %Multiply the hill fraction term by N0 and N1 respectively.
        entra.par(13) = entra.expVal;                                                    %l1
    end
    
    %Calculate k0 and k1 using steady state formula from CellCycle ref.
    [K0,K1,GA]    = NssCalc(entra.par(1),entra.par(2),entra.par(3),entra.par(4),...
        entra.par(5),entra.par(6),entra.N0,entra.N1,entra.N2,entra.par(11)); %Using steady state formula from CellCycle ref.
    entra.par(8)  = K0;                                                             %k0
    entra.par(9)  = K1;                                                             %k1
    entra.par(7)  = GA;                                                             %gamma 
    
    %Create figure
    entra = plotFigure(entra); %Go to function: plotFigure
    
    if entra.sensCalc == 1
        entra = colonSens(entra); %Calculate the sensitivities
    end
    
end

%%
%Create figure
function entra = plotFigure(entra)
    
    soluciones = length(entra.del);
    
    if soluciones <= 3
            filas    = 1;
            columnas = soluciones; 
    else            
            filas    = 2;
            columnas = ceil(soluciones/2);
    end
    
    for n = 1:soluciones
        entra.delays = [entra.del(n) entra.del(n)];
        entra.n      = n;
        delay1       = repmat(entra.init(1),entra.neqs,1);
        delay2       = repmat(entra.init(2),entra.neqs,1);
        delay3       = [delay1;delay2];
        delay4       = zeros(entra.neqs*2,1);
%         options      = odeset('NonNegative',1:3);
        [t,y]        = ode15s(@eqs,entra.tspan,[entra.init;delay3],[],entra);
        t            = t/24;
        entra.t      = t;
        entra.y      = y(:,1:3);
        
        if entra.figura == 1
            figure(entra.f)
            subplot(filas,columnas,entra.n)

            try
                y = y(:,entra.paintVar);
                y(y < 0) =0;
                subplot(1,2,1)
                plot(t,y,entra.color);
                set(gca,'fontsize',16)
                ylim([0 max(max(y))])
                legend(entra.vars{entra.paintVar},'Location','SouthEast')
                subplot(1,2,2),plot(t,sum(y,2))
                legend('Total cells','Location','SouthEast')
                ylim([0 max(sum(y,2))])
            catch
                y(y < 0) =0;
                plot(t,sum(y(:,1:3),2),entra.color)
                set(gca,'fontsize',16)
%                 ylim([0 max(sum(y,2))])
%                 legend('Total cells','Location','SouthEast')
                hold on
            end

            xlabel('time (days)','FontSize',15)
            ylabel('Population cells','FontSize',15)
            a      = 'Time delay =';
            b      = num2str(entra.del(n));
            c      = 'hours';
            titulo = sprintf('%s %s %s',a,b,c);

            if entra.delay == 1
%                 title(titulo,'FontSize',13)
            end

        end
        
    end
    
end

%%
%Calculate k0 and k1 using steady state formula from CellCycle ref. 
function [K0,K1,GA] = NssCalc(a1,a2,a3,b1,b2,b3,n0,n1,n2,M1)
  
    warning('off')
    syms N1 k1 BETA m1 alpha1 alpha3 N0
    K0 = (a3 - a2 - a1)/n0;
    eq = (1/(2*(k1-BETA*m1)))*(BETA+m1*(N0*(alpha3-alpha1))+...
        sqrt((BETA-m1*N0*(alpha3-alpha1))^2+4*k1*N0*(alpha3-alpha1)))-N1;
    K1 = solve(eq,'k1');
    K1 = subs(K1,N0,n0);
    K1 = subs(K1,alpha3,a3);
    K1 = subs(K1,alpha1,a1);
    K1 = subs(K1,BETA,b3 - b2 - b1);
    K1 = subs(K1,N1,n1);
    K1 = subs(K1,m1,M1);
    K1 = double(K1);
    GA = (1/n2)*(b2*n1 + (K1*(n1^2))/(1 + M1*n1));

end

function entra = colonSens(entra)

    for n = 1:length(entra.del)
        entra.n = n;
        entra.delays = [entra.del(n) entra.del(n)];
%         options      = odeset('NonNegative',1:3);
        [~,y0]       = ode15s(@eqs,entra.tspan,[entra.init;zeros(entra.neqs*2,1)],[],entra);
        par0         = entra.par;
        sensM        = zeros(length(par0),length(entra.vars));%
        sensS        = zeros(length(par0),length(entra.vars));%
        samples      = 20;
        step         = 1e-3;
        sensKind     = 'Dynamic sensitivity, number of cells respecting parameter change';

        for m = 1:length(par0)
            sensSample = zeros(samples,3);
            fprintf('Condition: %g/%g | Sol: %g/%g | Par: %g/%g\n',entra.f,entra.condiciones,n,length(entra.del),m,length(par0))

            for p = 1:samples
                entra.par(m)    = par0(m);
                entra.par(m)    = par0(m) + (rand*2 - 1)*step*par0(m);
%                 options         = odeset('NonNegative',1:3);
                [~,y1]          = ode15s(@eqs,entra.tspan,[entra.init;zeros(entra.neqs*2,1)],[],entra);
                sensSample(p,:) = ((log(sum(y1(:,1:3),1)) - log(sum(y0(:,1:3),1)))/(log(entra.par(m)) - log(par0(m))))/(entra.tspan(end))/24;
            end
            
            sensM(sensM./sensM ~= 1) = 0;
            sensS(sensS./sensS ~= 1) = 0;
            sensM(m,:)               = mean(sensSample,1);
            sensS(m,:)               = std(sensSample,[],1);
        end

        entra.sens               = mean(mean(abs(sensM)));
        entra.par                = par0;
        titulo1                  = strcat('Mean Sensitivities.Condition.',num2str(entra.f),'.Solution.',num2str(n));
        titulo2                  = strcat('Error Sensitivities.Condition.',num2str(entra.f),'.Solution.',num2str(n));
        archivo                  = sprintf('sensColon_Cond_%d_Sol_%d.mat',entra.f,n);
        sensM(sensM./sensM ~= 1) = 0;
        save(archivo,'sensM','sensS','samples','step','sensKind','entra')
        figure(100*entra.f + n)
        imagesc(abs(sensM))
        set(gca,'YTick',1:length(entra.names),'XTick',1:length(entra.vars),'YTickLabel',entra.names,'XTickLabel',entra.vars)
        colorbar
        colormap('copper')
        title(titulo1)
        figure(1000*entra.f + n)
        imagesc(abs(sensS))
        set(gca,'YTick',1:length(entra.names),'XTick',1:length(entra.vars),'YTickLabel',entra.names,'XTickLabel',entra.vars)
        colorbar
        colormap('copper')
        title(titulo2)
        figure(10000*entra.f + n)
    end

end
